@@ -33,6 +33,9 @@
  */
 /* copyright --> */
 #include "PeerReceiveHandshakeCommand.h"
+
+#include <cstring>
+
 #include "PeerConnection.h"
 #include "DownloadEngine.h"
 #include "BtHandshakeMessage.h"
@@ -45,20 +48,16 @@
 #include "PeerStorage.h"
 #include "PieceStorage.h"
 #include "BtRuntime.h"
-#include "BtAnnounce.h"
-#include "BtProgressInfoFile.h"
 #include "BtConstants.h"
 #include "message.h"
-#include "Socket.h"
+#include "SocketCore.h"
 #include "Logger.h"
+#include "LogFactory.h"
 #include "prefs.h"
 #include "Option.h"
 #include "RequestGroupMan.h"
-#include "StringFormat.h"
+#include "fmt.h"
 #include "RequestGroup.h"
-#include "ServerStatMan.h"
-#include "FileAllocationEntry.h"
-#include "CheckIntegrityEntry.h"
 
 namespace aria2 {
 
@@ -66,14 +65,18 @@ PeerReceiveHandshakeCommand::PeerReceiveHandshakeCommand
 (cuid_t cuid,
  const SharedHandle<Peer>& peer,
  DownloadEngine* e,
- const SocketHandle& s,
+ const SharedHandle<SocketCore>& s,
  const SharedHandle<PeerConnection>& peerConnection)
-  :
-  PeerAbstractCommand(cuid, peer, e, s),
-  peerConnection_(peerConnection)
+  : PeerAbstractCommand(cuid, peer, e, s),
+    peerConnection_(peerConnection)
 {
-  if(peerConnection_.isNull()) {
-    peerConnection_.reset(new PeerConnection(cuid, getSocket()));
+  if(peerConnection_) {
+    if(peerConnection_->getBufferLength() > 0) {
+      setStatus(Command::STATUS_ONESHOT_REALTIME);
+      getDownloadEngine()->setNoWait(true);
+    }
+  } else {
+    peerConnection_.reset(new PeerConnection(cuid, getPeer(), getSocket()));
   }
 }
 
@@ -87,39 +90,45 @@ bool PeerReceiveHandshakeCommand::exitBeforeExecute()
 
 bool PeerReceiveHandshakeCommand::executeInternal()
 {
-  unsigned char data[BtHandshakeMessage::MESSAGE_LENGTH];
-  size_t dataLength = BtHandshakeMessage::MESSAGE_LENGTH;
-  // ignore return value. The received data is kept in PeerConnection object
-  // because of peek = true.
-  peerConnection_->receiveHandshake(data, dataLength, true);
-  // To handle tracker's NAT-checking feature
-  if(dataLength >= 48) {
+  // Handle tracker's NAT-checking feature
+  if(peerConnection_->getBufferLength() < 48) {
+    size_t dataLength = 0;
+    // Ignore return value. The received data is kept in
+    // PeerConnection object because of peek = true.
+    peerConnection_->receiveHandshake(0, dataLength, true);
+  }
+  if(peerConnection_->getBufferLength() >= 48) {
+    const unsigned char* data = peerConnection_->getBuffer();
     // check info_hash
-    std::string infoHash = std::string(&data[28], &data[28+INFO_HASH_LENGTH]);
+    std::string infoHash(&data[28], &data[28+INFO_HASH_LENGTH]);
 
     SharedHandle<DownloadContext> downloadContext =
       getDownloadEngine()->getBtRegistry()->getDownloadContext(infoHash);
-    if(downloadContext.isNull()) {
+    if(!downloadContext) {
       throw DL_ABORT_EX
-        (StringFormat("Unknown info hash %s",
-                      util::toHex(infoHash).c_str()).str());
+        (fmt("Unknown info hash %s",
+             util::toHex(infoHash).c_str()));
     }
-    BtObject btObject = getDownloadEngine()->getBtRegistry()->get
+    const SharedHandle<BtObject>& btObject =
+      getDownloadEngine()->getBtRegistry()->get
       (downloadContext->getOwnerRequestGroup()->getGID());
-    SharedHandle<BtRuntime> btRuntime = btObject.btRuntime_;
-    SharedHandle<PieceStorage> pieceStorage = btObject.pieceStorage_;
-    SharedHandle<PeerStorage> peerStorage = btObject.peerStorage_;
+    const SharedHandle<BtRuntime>& btRuntime = btObject->btRuntime;
+    const SharedHandle<PieceStorage>& pieceStorage = btObject->pieceStorage;
+    const SharedHandle<PeerStorage>& peerStorage = btObject->peerStorage;
     if(!btRuntime->ready()) {
       throw DL_ABORT_EX
-        (StringFormat("Unknown info hash %s",
-                      util::toHex(infoHash).c_str()).str());
+        (fmt("Unknown info hash %s",
+             util::toHex(infoHash).c_str()));
     }
-
-    TransferStat tstat =
-      downloadContext->getOwnerRequestGroup()->calculateStat();
-    const unsigned int maxDownloadLimit =
+    if(btRuntime->isHalt()) {
+      A2_LOG_DEBUG("Info hash found but the download is over."
+                   " Dropping connection.");
+      return true;
+    }
+    NetStat& stat = downloadContext->getNetStat();
+    const int maxDownloadLimit =
       downloadContext->getOwnerRequestGroup()->getMaxDownloadSpeedLimit();
-    unsigned int thresholdSpeed =
+    int thresholdSpeed =
       downloadContext->getOwnerRequestGroup()->
       getOption()->getAsInt(PREF_BT_REQUEST_PEER_SPEED_LIMIT);
     if(maxDownloadLimit > 0) {
@@ -127,10 +136,12 @@ bool PeerReceiveHandshakeCommand::executeInternal()
     }
 
     if((!pieceStorage->downloadFinished() &&
-        tstat.getDownloadSpeed() < thresholdSpeed) ||
+        stat.calculateDownloadSpeed() < thresholdSpeed) ||
        btRuntime->lessThanMaxPeers()) {
-      if(peerStorage->addPeer(getPeer())) {
-        getPeer()->usedBy(getCuid());
+      // TODO addPeer and checkoutPeer must be "atomic", in a sense
+      // that the added peer must be checked out.
+      if(peerStorage->addPeer(getPeer()) &&
+         peerStorage->checkoutPeer(getCuid())) {
         PeerInteractionCommand* command =
           new PeerInteractionCommand
           (getCuid(),
@@ -144,11 +155,9 @@ bool PeerReceiveHandshakeCommand::executeInternal()
            PeerInteractionCommand::RECEIVER_WAIT_HANDSHAKE,
            peerConnection_);
         getDownloadEngine()->addCommand(command);
-        if(getLogger()->debug()) {
-          getLogger()->debug(MSG_INCOMING_PEER_CONNECTION,
-                             util::itos(getCuid()).c_str(),
-                             util::itos(getPeer()->usedBy()).c_str());
-        }
+        A2_LOG_DEBUG(fmt(MSG_INCOMING_PEER_CONNECTION,
+                         getCuid(),
+                         getPeer()->usedBy()));
       }
     }
     return true;
