@@ -34,6 +34,7 @@
 /* copyright --> */
 #include "PortEventPoll.h"
 
+#include <cerrno>
 #include <cstring>
 #include <algorithm>
 #include <numeric>
@@ -41,11 +42,14 @@
 #include "Command.h"
 #include "LogFactory.h"
 #include "Logger.h"
+#include "util.h"
+#include "fmt.h"
 
 namespace aria2 {
 
-PortEventPoll::KSocketEntry::KSocketEntry(sock_t s):
-  SocketEntry<KCommandEvent, KADNSEvent>(s) {}
+PortEventPoll::KSocketEntry::KSocketEntry(sock_t s)
+  : SocketEntry<KCommandEvent, KADNSEvent>(s)
+{}
 
 int accumulateEvent(int events, const PortEventPoll::KEvent& event)
 {
@@ -72,10 +76,9 @@ PortEventPoll::A2PortEvent PortEventPoll::KSocketEntry::getEvents()
   return portEvent;
 }
 
-PortEventPoll::PortEventPoll():
-  portEventsSize_(PORT_EVENTS_SIZE),
-  portEvents_(new port_event_t[portEventsSize_]),
-  logger_(LogFactory::getInstance())
+PortEventPoll::PortEventPoll()
+  : portEventsSize_(PORT_EVENTS_SIZE),
+    portEvents_(new port_event_t[portEventsSize_])
 {
   port_ = port_create();
 }
@@ -83,11 +86,12 @@ PortEventPoll::PortEventPoll():
 PortEventPoll::~PortEventPoll()
 {
   if(port_ != -1) {
-    int r;
-    while((r = close(port_)) == -1 && errno == EINTR);
+    int r = close(port_);
+    int errNum = errno;
     if(r == -1) {
-      logger_->error("Error occurred while closing port %d: %s",
-                     port_, strerror(errno));
+      A2_LOG_ERROR(fmt("Error occurred while closing port %d: %s",
+                       port_,
+                       util::safeStrerror(errNum).c_str()));
     }
   }
   delete [] portEvents_;
@@ -111,29 +115,32 @@ void PortEventPoll::poll(const struct timeval& tv)
   if(res == 0 ||
      (res == -1 && (errno == ETIME || errno == EINTR) &&
       portEvents_[0].portev_user != (void*)-1)) {
-    if(logger_->debug()) {
-      logger_->debug("nget=%u", nget);
-    }
+    A2_LOG_DEBUG(fmt("nget=%u", nget));
     for(uint_t i = 0; i < nget; ++i) {
       const port_event_t& pev = portEvents_[i];
       KSocketEntry* p = reinterpret_cast<KSocketEntry*>(pev.portev_user);
       p->processEvents(pev.portev_events);
       int r = port_associate(port_, PORT_SOURCE_FD, pev.portev_object,
                              p->getEvents().events, p);
+      int errNum = errno;
       if(r == -1) {
-        logger_->error("port_associate failed for file descriptor %d: cause %s",
-                       pev.portev_object, strerror(errno));
+        A2_LOG_INFO(fmt("port_associate failed for file descriptor %d:"
+                        " cause %s",
+                        pev.portev_object,
+                        util::safeStrerror(errNum).c_str()));
       }
     }
+  } else if(res == -1) {
+    int errNum = errno;
+    A2_LOG_INFO(fmt("port_getn error: %s", util::safeStrerror(errNum).c_str()));
   }
 #ifdef ENABLE_ASYNC_DNS
   // It turns out that we have to call ares_process_fd before ares's
   // own timeout and ares may create new sockets or closes socket in
   // their API. So we call ares_process_fd for all ares_channel and
   // re-register their sockets.
-  for(std::deque<SharedHandle<KAsyncNameResolverEntry> >::iterator i =
-        nameResolverEntries_.begin(), eoi = nameResolverEntries_.end();
-      i != eoi; ++i) {
+  for(KAsyncNameResolverEntrySet::iterator i = nameResolverEntries_.begin(),
+        eoi = nameResolverEntries_.end(); i != eoi; ++i) {
     (*i)->processTimeout();
     (*i)->removeSocketEvents(this);
     (*i)->addSocketEvents(this);
@@ -168,14 +175,15 @@ bool PortEventPoll::addEvents(sock_t socket,
                               const PortEventPoll::KEvent& event)
 {
   SharedHandle<KSocketEntry> socketEntry(new KSocketEntry(socket));
-  std::deque<SharedHandle<KSocketEntry> >::iterator i =
-    std::lower_bound(socketEntries_.begin(), socketEntries_.end(), socketEntry);
+  KSocketEntrySet::iterator i = socketEntries_.lower_bound(socketEntry);
   int r = 0;
-  if(i != socketEntries_.end() && (*i) == socketEntry) {
+  int errNum = 0;
+  if(i != socketEntries_.end() && *(*i) == *socketEntry) {
     event.addSelf(*i);
     A2PortEvent pv = (*i)->getEvents();
     r = port_associate(port_, PORT_SOURCE_FD, (*i)->getSocket(),
                        pv.events, pv.socketEntry);
+    errNum = r;
   } else {
     socketEntries_.insert(i, socketEntry);
     if(socketEntries_.size() > portEventsSize_) {
@@ -187,12 +195,11 @@ bool PortEventPoll::addEvents(sock_t socket,
     A2PortEvent pv = socketEntry->getEvents();
     r = port_associate(port_, PORT_SOURCE_FD, socketEntry->getSocket(),
                        pv.events, pv.socketEntry);
+    errNum = r;
   }
   if(r == -1) {
-    if(logger_->debug()) {
-      logger_->debug("Failed to add socket event %d:%s",
-                     socket, strerror(errno));
-    }
+    A2_LOG_DEBUG(fmt("Failed to add socket event %d:%s",
+                     socket, util::safeStrerror(errNum).c_str()));
     return false;
   } else {
     return true;
@@ -218,32 +225,31 @@ bool PortEventPoll::deleteEvents(sock_t socket,
                                  const PortEventPoll::KEvent& event)
 {
   SharedHandle<KSocketEntry> socketEntry(new KSocketEntry(socket));
-  std::deque<SharedHandle<KSocketEntry> >::iterator i =
-    std::lower_bound(socketEntries_.begin(), socketEntries_.end(), socketEntry);
-  if(i != socketEntries_.end() && (*i) == socketEntry) {
+  KSocketEntrySet::iterator i = socketEntries_.find(socketEntry);
+  if(i == socketEntries_.end()) {
+    A2_LOG_DEBUG(fmt("Socket %d is not found in SocketEntries.", socket));
+    return false;
+  } else {
     event.removeSelf(*i);
     int r = 0;
+    int errNum = 0;
     if((*i)->eventEmpty()) {
       r = port_dissociate(port_, PORT_SOURCE_FD, (*i)->getSocket());
+      errNum = errno;
       socketEntries_.erase(i);
     } else {
       A2PortEvent pv = (*i)->getEvents();
       r = port_associate(port_, PORT_SOURCE_FD, (*i)->getSocket(),
                          pv.events, pv.socketEntry);
+      errNum = errno;
     }
     if(r == -1) {
-      if(logger_->debug()) {
-        logger_->debug("Failed to delete socket event:%s", strerror(errno));
-      }
+      A2_LOG_DEBUG(fmt("Failed to delete socket event:%s",
+                       util::safeStrerror(errNum).c_str()));
       return false;
     } else {
       return true;
     }
-  } else {
-    if(logger_->debug()) {
-      logger_->debug("Socket %d is not found in SocketEntries.", socket);
-    }
-    return false;
   }
 }
 
@@ -268,10 +274,9 @@ bool PortEventPoll::addNameResolver
 {
   SharedHandle<KAsyncNameResolverEntry> entry
     (new KAsyncNameResolverEntry(resolver, command));
-  std::deque<SharedHandle<KAsyncNameResolverEntry> >::iterator itr =
-    std::find(nameResolverEntries_.begin(), nameResolverEntries_.end(), entry);
+  KAsyncNameResolverEntrySet::iterator itr = nameResolverEntries_.find(entry);
   if(itr == nameResolverEntries_.end()) {
-    nameResolverEntries_.push_back(entry);
+    nameResolverEntries_.insert(entry);
     entry->addSocketEvents(this);
     return true;
   } else {
@@ -284,8 +289,7 @@ bool PortEventPoll::deleteNameResolver
 {
   SharedHandle<KAsyncNameResolverEntry> entry
     (new KAsyncNameResolverEntry(resolver, command));
-  std::deque<SharedHandle<KAsyncNameResolverEntry> >::iterator itr =
-    std::find(nameResolverEntries_.begin(), nameResolverEntries_.end(), entry);
+  KAsyncNameResolverEntrySet::iterator itr = nameResolverEntries_.find(entry);
   if(itr == nameResolverEntries_.end()) {
     return false;
   } else {
